@@ -8,6 +8,7 @@ from django.contrib.admin.widgets import FilteredSelectMultiple
 from django.contrib.auth.models import Permission
 from django.db.models import Q
 from django.db.models.fields.files import FieldFile
+from django.template.defaultfilters import urlencode
 from django.urls import reverse
 from django.utils.html import format_html
 from django.utils.safestring import mark_safe
@@ -32,6 +33,8 @@ from NEMO.models import (
 	BadgeReader,
 	BuddyRequest,
 	BuddyRequestMessage,
+	Closure,
+	ClosureTime,
 	Comment,
 	Configuration,
 	ConfigurationHistory,
@@ -50,7 +53,6 @@ from NEMO.models import (
 	MembershipHistory,
 	News,
 	Notification,
-	PhysicalAccessException,
 	PhysicalAccessLevel,
 	PhysicalAccessLog,
 	Project,
@@ -61,12 +63,17 @@ from NEMO.models import (
 	SafetyIssue,
 	ScheduledOutage,
 	ScheduledOutageCategory,
+	StaffAbsence,
+	StaffAbsenceType,
+	StaffAvailability,
 	StaffCharge,
 	Task,
 	TaskCategory,
 	TaskHistory,
 	TaskImages,
 	TaskStatus,
+	TemporaryPhysicalAccess,
+	TemporaryPhysicalAccessRequest,
 	Tool,
 	ToolDocuments,
 	ToolUsageCounter,
@@ -79,6 +86,7 @@ from NEMO.models import (
 	record_local_many_to_many_changes,
 	record_remote_many_to_many_changes_and_save,
 )
+from NEMO.utilities import format_daterange
 from NEMO.widgets.dynamic_form import (
 	DynamicForm,
 	PostUsageFloatFieldQuestion,
@@ -179,6 +187,7 @@ class ToolAdminForm(forms.ModelForm):
 					self.add_error("_policy_off_start_time", "Start time must be specified")
 				if not policy_off_end_time:
 					self.add_error("_policy_off_end_time", "End time must be specified")
+		return cleaned_data
 
 
 class ToolDocumentsInline(admin.TabularInline):
@@ -322,6 +331,7 @@ class AreaAdmin(DraggableMPTTAdmin):
 		"requires_reservation",
 		"maximum_capacity",
 		"reservation_warning",
+		"buddy_system_allowed",
 		"id",
 	)
 	fieldsets = (
@@ -580,6 +590,7 @@ class ReservationQuestionsForm(forms.ModelForm):
 			except Exception:
 				error_info = sys.exc_info()
 				self.add_error("questions", error_info[0].__name__ + ": " + str(error_info[1]))
+		return cleaned_data
 
 
 @register(ReservationQuestions)
@@ -656,11 +667,12 @@ class InterlockCardAdminForm(forms.ModelForm):
 	def clean(self):
 		if any(self.errors):
 			return
-		super(InterlockCardAdminForm, self).clean()
-		category = self.cleaned_data["category"]
+		cleaned_data = super().clean()
+		category = cleaned_data["category"]
 		from NEMO import interlocks
 
 		interlocks.get(category, False).clean_interlock_card(self)
+		return cleaned_data
 
 
 @register(InterlockCard)
@@ -677,11 +689,12 @@ class InterlockAdminForm(forms.ModelForm):
 	def clean(self):
 		if any(self.errors):
 			return
-		super(InterlockAdminForm, self).clean()
+		cleaned_data = super().clean()
 		from NEMO import interlocks
 
 		category = self.cleaned_data["card"].category
 		interlocks.get(category, False).clean_interlock(self)
+		return cleaned_data
 
 
 @register(Interlock)
@@ -840,6 +853,7 @@ class UserAdminForm(forms.ModelForm):
 					"is_service_personnel": "A user cannot be both staff and service personnel. Please choose one or the other.",
 				}
 			)
+		return cleaned_data
 
 
 @register(User)
@@ -864,6 +878,7 @@ class UserAdmin(admin.ModelAdmin):
 				"fields": (
 					"is_active",
 					"is_staff",
+					"is_facility_manager",
 					"is_technician",
 					"is_service_personnel",
 					"is_superuser",
@@ -899,6 +914,7 @@ class UserAdmin(admin.ModelAdmin):
 		"is_staff",
 		"is_technician",
 		"is_service_personnel",
+		"is_facility_manager",
 		"is_superuser",
 		"date_joined",
 		"last_login",
@@ -907,6 +923,7 @@ class UserAdmin(admin.ModelAdmin):
 		"is_active",
 		"domain",
 		"is_staff",
+		"is_facility_manager",
 		"is_technician",
 		"is_service_personnel",
 		"is_superuser",
@@ -989,6 +1006,7 @@ class AlertAdmin(admin.ModelAdmin):
 		"expired",
 		"deleted",
 	)
+	list_filter = ("category", "dismissible", "expired", "deleted")
 	form = AlertAdminForm
 
 
@@ -1006,51 +1024,80 @@ class PhysicalAccessLevelForm(forms.ModelForm):
 		widget=FilteredSelectMultiple(verbose_name="Users", is_stacked=False),
 	)
 
-	physical_access_exceptions = forms.ModelMultipleChoiceField(
-		queryset=PhysicalAccessException.objects.all(),
+	closures = forms.ModelMultipleChoiceField(
+		queryset=Closure.objects.all(),
 		required=False,
-		widget=FilteredSelectMultiple(verbose_name="Physical Access Exceptions", is_stacked=False),
+		widget=FilteredSelectMultiple(verbose_name="Closures", is_stacked=False),
 	)
 
 	def __init__(self, *args, **kwargs):
 		super().__init__(*args, **kwargs)
 		if self.instance.pk:
 			self.fields["authorized_users"].initial = self.instance.user_set.all()
-			self.fields["physical_access_exceptions"].initial = self.instance.physicalaccessexception_set.all()
+			self.fields["closures"].initial = self.instance.closure_set.all()
 
 	def clean(self):
-		schedule = self.cleaned_data.get("schedule")
+		cleaned_data = super().clean()
+		schedule = cleaned_data.get("schedule")
 		if schedule == PhysicalAccessLevel.Schedule.WEEKDAYS:
-			start_date = self.cleaned_data.get("weekdays_start_time")
-			end_date = self.cleaned_data.get("weekdays_end_time")
+			start_date = cleaned_data.get("weekdays_start_time")
+			end_date = cleaned_data.get("weekdays_end_time")
 			if not start_date:
 				self.add_error("weekdays_start_time", "Start time is required for weekdays.")
 			if not end_date:
 				self.add_error("weekdays_end_time", "End time is required for weekdays.")
 		else:
-			self.cleaned_data["weekdays_start_time"] = None
-			self.cleaned_data["weekdays_end_time"] = None
-		return self.cleaned_data
+			cleaned_data["weekdays_start_time"] = None
+			cleaned_data["weekdays_end_time"] = None
+		return cleaned_data
 
 
 @register(PhysicalAccessLevel)
 class PhysicalAccessLevelAdmin(admin.ModelAdmin):
 	form = PhysicalAccessLevelForm
-	list_display = ("name", "area", "get_schedule_display_with_times", "allow_staff_access")
-	list_filter = (("area", TreeRelatedFieldListFilter),)
+	list_display = ("name", "area", "get_schedule_display_with_times", "allow_staff_access", "allow_user_request")
+	list_filter = (("area", TreeRelatedFieldListFilter), "allow_staff_access", "allow_user_request")
 
 	def save_model(self, request, obj, form, change):
 		"""
 		Explicitly record any membership changes.
 		"""
 		record_remote_many_to_many_changes_and_save(request, obj, form, change, "authorized_users", super().save_model)
-		if "physical_access_exceptions" in form.changed_data:
-			obj.physicalaccessexception_set.set(form.cleaned_data["physical_access_exceptions"])
+		if "closures" in form.changed_data:
+			obj.closure_set.set(form.cleaned_data["closures"])
 
 
-class PhysicalAccessExceptionAdminForm(forms.ModelForm):
+class ClosureTimeInline(admin.TabularInline):
+	model = ClosureTime
+	min_num = 1
+
+
+class ClosureAdminForm(forms.ModelForm):
 	class Meta:
-		model = PhysicalAccessException
+		model = Closure
+		fields = "__all__"
+
+	class Media:
+		js = ("admin/time_options_override.js",)
+
+
+@register(Closure)
+class ClosureAdmin(admin.ModelAdmin):
+	inlines = [ClosureTimeInline]
+	form = ClosureAdminForm
+	list_display = ("name", "alert_days_before", "get_times_display", "staff_absent", "notify_managers_last_occurrence")
+	filter_horizontal = ("physical_access_levels",)
+	list_filter = ("physical_access_levels__area", "staff_absent", "notify_managers_last_occurrence")
+
+	def get_times_display(self, closure: Closure) -> str:
+		return mark_safe("<br>".join([format_daterange(ct.start_time, ct.end_time, dt_format="SHORT_DATETIME_FORMAT", d_format="SHORT_DATE_FORMAT", date_separator=" ", time_separator=" - ") for ct in ClosureTime.objects.filter(closure=closure)]))
+
+	get_times_display.short_description = 'Times'
+
+
+class TemporaryPhysicalAccessAdminForm(forms.ModelForm):
+	class Meta:
+		model = TemporaryPhysicalAccess
 		fields = "__all__"
 
 	class Media:
@@ -1064,14 +1111,51 @@ class PhysicalAccessExceptionAdminForm(forms.ModelForm):
 		end_time = cleaned_data.get("end_time")
 		if end_time <= start_time:
 			self.add_error("end_time", "The end time must be later than the start time")
+		return cleaned_data
 
 
-@register(PhysicalAccessException)
-class PhysicalAccessExceptionAdmin(admin.ModelAdmin):
-	form = PhysicalAccessExceptionAdminForm
-	list_display = ("name", "start_time", "end_time")
-	filter_horizontal = ("physical_access_levels",)
-	list_filter = ("physical_access_levels__area",)
+@register(TemporaryPhysicalAccess)
+class TemporaryPhysicalAccessAdmin(admin.ModelAdmin):
+	list_display = ("id", "user", "start_time", "end_time", "get_area_name", "get_schedule_display_with_times")
+	list_filter = ("physical_access_level", "physical_access_level__area", "end_time", "start_time")
+	form = TemporaryPhysicalAccessAdminForm
+
+	def get_area_name(self, tpa: TemporaryPhysicalAccess) -> str:
+		return tpa.physical_access_level.area.name
+
+	get_area_name.admin_order_field = 'physical_access_level__area'
+	get_area_name.short_description = 'Area'
+
+	def get_schedule_display_with_times(self, tpa: TemporaryPhysicalAccess) -> str:
+		return tpa.physical_access_level.get_schedule_display_with_times()
+
+	get_schedule_display_with_times.short_description = 'Schedule'
+
+
+class TemporaryPhysicalAccessRequestFormAdmin(forms.ModelForm):
+	class Meta:
+		model = TemporaryPhysicalAccessRequest
+		fields = "__all__"
+
+
+@register(TemporaryPhysicalAccessRequest)
+class TemporaryPhysicalAccessRequestAdmin(admin.ModelAdmin):
+	form = TemporaryPhysicalAccessRequestFormAdmin
+	list_display = ("creator", "creation_time", "other_users_display", "start_time", "end_time", "physical_access_level", "status_display", "reviewer", "deleted")
+	list_filter = ("status", "deleted")
+	filter_horizontal = ("other_users",)
+
+	def other_users_display(self, access_request: TemporaryPhysicalAccessRequest):
+		return mark_safe("<br>".join([u.username for u in access_request.other_users.all()]))
+
+	other_users_display.admin_order_field = "other_users"
+	other_users_display.short_description = "Buddies"
+
+	def status_display(self, access_request: TemporaryPhysicalAccessRequest):
+		return access_request.get_status_display()
+
+	status_display.admin_order_field = "status"
+	status_display.short_description = "Status"
 
 
 @register(ContactInformationCategory)
@@ -1122,6 +1206,7 @@ class NewsAdmin(admin.ModelAdmin):
 @register(Notification)
 class NotificationAdmin(admin.ModelAdmin):
 	list_display = ("id", "user", "expiration", "content_type", "object_id")
+	list_filter = (("content_type", admin.RelatedOnlyFieldListFilter),)
 
 
 @register(BadgeReader)
@@ -1203,6 +1288,29 @@ class BuddyRequestMessageAdmin(admin.ModelAdmin):
 	link_to_buddy_request.short_description = "BUDDY REQUEST"
 
 
+@register(StaffAbsenceType)
+class StaffAbsenceTypeAdmin(admin.ModelAdmin):
+	list_display = ("name", "description")
+
+
+@register(StaffAvailability)
+class StaffAvailabilityAdmin(admin.ModelAdmin):
+	list_display = ("staff_member", "category", "start_time", "end_time", *StaffAvailability.DAYS)
+	list_filter = ("category", *StaffAvailability.DAYS)
+
+	def formfield_for_foreignkey(self, db_field, request, **kwargs):
+		""" We only want active users here """
+		if db_field.name == "staff_member":
+			kwargs["queryset"] = User.objects.filter(is_active=True)
+		return super().formfield_for_foreignkey(db_field, request, **kwargs)
+
+
+@register(StaffAbsence)
+class StaffAbsenceAdmin(admin.ModelAdmin):
+	list_display = ("creation_time", "staff_member", "absence_type", "full_day", "start_date", "end_date")
+	list_filter = ("staff_member", "absence_type", "start_date", "end_date", "creation_time")
+
+
 @register(EmailLog)
 class EmailLogAdmin(admin.ModelAdmin):
 	list_display = ["id", "category", "sender", "to", "subject", "when", "ok"]
@@ -1212,7 +1320,10 @@ class EmailLogAdmin(admin.ModelAdmin):
 	date_hierarchy = "when"
 
 	def content_preview(self, obj):
-		return mark_safe(obj.content)
+		if obj.content:
+			return mark_safe(f'<div style="position: relative; display: block; overflow: hidden; padding-bottom: 75%"><iframe style="position: absolute; width:100%; height:100%; border:none" src="data:text/html,{urlencode(obj.content)}"></iframe></div>')
+		else:
+			return ""
 
 	def has_delete_permission(self, request, obj=None):
 		return False
