@@ -8,10 +8,10 @@ from django.forms import (
 	ChoiceField,
 	DateField,
 	Form,
+	ImageField,
 	IntegerField,
 	ModelChoiceField,
 	ModelForm,
-	ImageField,
 )
 from django.forms.utils import ErrorDict
 from django.utils import timezone
@@ -19,22 +19,25 @@ from django.utils import timezone
 from NEMO.models import (
 	Account,
 	Alert,
+	AlertCategory,
+	BuddyRequest,
 	Comment,
 	Consumable,
 	ConsumableWithdraw,
 	Project,
+	ReservationItemType,
 	SafetyIssue,
 	ScheduledOutage,
+	StaffAbsence,
 	Task,
 	TaskCategory,
+	TaskImages,
+	TemporaryPhysicalAccessRequest,
 	User,
 	UserPreferences,
-	TaskImages,
-	AlertCategory,
-	ReservationItemType,
-	BuddyRequest,
 )
-from NEMO.utilities import bootstrap_primary_color, format_datetime
+from NEMO.utilities import bootstrap_primary_color, format_datetime, quiet_int
+from NEMO.views.customization import get_customization
 
 
 class UserForm(ModelForm):
@@ -99,16 +102,17 @@ class TaskForm(ModelForm):
 	def clean(self):
 		if any(self.errors):
 			return
-		super(TaskForm, self).clean()
-		action = self.cleaned_data["action"]
+		cleaned_data = super().clean()
+		action = cleaned_data["action"]
 		if action == "create":
-			if not self.cleaned_data["description"]:
+			if not cleaned_data["description"]:
 				raise ValidationError("You must describe the problem.")
 		if action == "resolve":
 			if self.instance.cancelled or self.instance.resolved:
 				raise ValidationError(
 					"This task can't be resolved because it is marked as 'cancelled' or 'resolved' already."
 				)
+		return cleaned_data
 
 	def save(self, commit=True):
 		instance = super(TaskForm, self).save(commit=False)
@@ -222,7 +226,7 @@ class SafetyIssueUpdateForm(ModelForm):
 		if progress_type == "updated" and self.cleaned_data["update"]:
 			progress = (
 					"On "
-					+ format_datetime(timezone.now())
+					+ format_datetime()
 					+ " "
 					+ self.user.get_full_name()
 					+ " updated this issue:\n"
@@ -273,23 +277,24 @@ class ConsumableWithdrawForm(ModelForm):
 	def clean(self):
 		if any(self.errors):
 			return
-		super(ConsumableWithdrawForm, self).clean()
-		quantity = self.cleaned_data["quantity"]
-		consumable = self.cleaned_data["consumable"]
+		cleaned_data = super().clean()
+		quantity = cleaned_data["quantity"]
+		consumable = cleaned_data["consumable"]
 		if quantity > consumable.quantity:
 			raise ValidationError(
 				'There are not enough "' + consumable.name + '". (The current quantity in stock is '
 				+ str(consumable.quantity)
 				+ "). Please order more as soon as possible."
 			)
-		customer = self.cleaned_data["customer"]
-		project = self.cleaned_data["project"]
+		customer = cleaned_data["customer"]
+		project = cleaned_data["project"]
 		if project not in customer.active_projects():
 			raise ValidationError(
 				"{} is not a member of the project {}. Users can only bill to projects they belong to.".format(
 					customer, project
 				)
 			)
+		return cleaned_data
 
 
 class ReservationAbuseForm(Form):
@@ -337,13 +342,17 @@ class EmailBroadcastForm(Form):
 	contents = CharField(required=False)
 	copy_me = BooleanField(initial=True)
 
-	audience = ChoiceField(choices=[("tool", "tool"), ("project", "project"), ("account", "account")])
-	selection = IntegerField()
+	audience = ChoiceField(choices=[("tool", "tool"), ("project", "project"), ("account", "account"), ("area", "area"), ("user", "user")])
+	selection = CharField(required=False)
+	no_type = BooleanField(initial=False, required=False)
 	only_active_users = BooleanField(initial=True)
 
 	def clean_title(self):
 		return self.cleaned_data["title"].upper()
 
+	def clean_selection(self):
+		return self.data.getlist('selection')
+
 
 class AlertForm(ModelForm):
 	class Meta:
@@ -384,6 +393,38 @@ class BuddyRequestForm(ModelForm):
 		model = BuddyRequest
 		fields = "__all__"
 
+	def clean(self):
+		if any(self.errors):
+			return
+		cleaned_data = super().clean()
+		start = cleaned_data.get("start")
+		end = cleaned_data.get("end")
+		if end < start:
+			self.add_error("end", "The end must be later than the start")
+		return cleaned_data
+
+
+class TemporaryPhysicalAccessRequestForm(ModelForm):
+	class Meta:
+		model = TemporaryPhysicalAccessRequest
+		fields = ["start_time", "end_time", "physical_access_level", "other_users", "description"]
+
+	def clean(self):
+		if any(self.errors):
+			return
+		cleaned_data = super().clean()
+		other_users = len(cleaned_data.get("other_users")) if "other_users" in cleaned_data else 0
+		minimum_total_users = quiet_int(get_customization("access_requests_minimum_users", 2))
+		if other_users < minimum_total_users -1:
+			self.add_error("other_users", f"You need at least {minimum_total_users-1} other {'buddy' if minimum_total_users == 2 else 'buddies'} for this request")
+		return cleaned_data
+
+
+class StaffAbsenceForm(ModelForm):
+	class Meta:
+		model = StaffAbsence
+		fields = "__all__"
+
 
 def nice_errors(form, non_field_msg="General form errors"):
 	result = ErrorDict()
