@@ -1,15 +1,15 @@
-from django.contrib.admin.views.decorators import staff_member_required
 from django.contrib.auth.decorators import login_required
 from django.http import HttpResponseBadRequest
-from django.shortcuts import render, get_object_or_404, redirect
+from django.shortcuts import get_object_or_404, redirect, render
 from django.views.decorators.http import require_GET, require_POST, require_http_methods
 
-from NEMO.forms import ProjectForm, AccountForm
-from NEMO.models import Account, Project, User, MembershipHistory, ActivityHistory
+from NEMO.decorators import staff_member_required
+from NEMO.forms import AccountForm, ProjectForm
+from NEMO.models import Account, ActivityHistory, MembershipHistory, Project, User
 from NEMO.views.pagination import SortedPaginator
 
 
-@staff_member_required(login_url=None)
+@staff_member_required
 @require_GET
 def accounts_and_projects(request):
 	all_accounts = Account.objects.all().order_by("name")
@@ -20,7 +20,7 @@ def accounts_and_projects(request):
 	return render(request, "accounts_and_projects/accounts_and_projects.html", dictionary)
 
 
-@staff_member_required(login_url=None)
+@staff_member_required
 @require_GET
 def select_accounts_and_projects(request, kind=None, identifier=None):
 	selected_project = None
@@ -43,7 +43,7 @@ def select_accounts_and_projects(request, kind=None, identifier=None):
 	return render(request, "accounts_and_projects/account_and_projects.html", dictionary)
 
 
-@staff_member_required(login_url=None)
+@staff_member_required
 @require_POST
 def toggle_active(request, kind, identifier):
 	if kind == "account":
@@ -62,7 +62,7 @@ def toggle_active(request, kind, identifier):
 	return redirect(request.META.get("HTTP_REFERER", "accounts_and_projects"))
 
 
-@staff_member_required(login_url=None)
+@staff_member_required
 @require_http_methods(["GET", "POST"])
 def create_project(request):
 	form = ProjectForm(request.POST or None)
@@ -86,7 +86,7 @@ def create_project(request):
 	return redirect("project", project.id)
 
 
-@staff_member_required(login_url=None)
+@staff_member_required
 @require_http_methods(["GET", "POST"])
 def create_account(request):
 	form = AccountForm(request.POST or None)
@@ -104,7 +104,7 @@ def create_account(request):
 	return redirect("account", account.id)
 
 
-@staff_member_required(login_url=None)
+@staff_member_required
 @require_POST
 def remove_user_from_project(request):
 	user = get_object_or_404(User, id=request.POST["user_id"])
@@ -121,7 +121,7 @@ def remove_user_from_project(request):
 	return render(request, "accounts_and_projects/users_for_project.html", dictionary)
 
 
-@staff_member_required(login_url=None)
+@staff_member_required
 @require_POST
 def add_user_to_project(request):
 	user = get_object_or_404(User, id=request.POST["user_id"])
