@@ -1,6 +1,5 @@
 from typing import List
 
-from django.utils import timezone
 from drf_renderer_xlsx.mixins import XLSXFileMixin
 from rest_framework import status
 from rest_framework.response import Response
@@ -37,6 +36,7 @@ from NEMO.serializers import (
 	UsageEventSerializer,
 	UserSerializer,
 )
+from NEMO.utilities import export_format_datetime
 from NEMO.views.api_billing import (
 	BillableItem,
 	BillingFilterForm,
@@ -48,8 +48,6 @@ from NEMO.views.api_billing import (
 	get_usage_events_for_billing,
 )
 
-date_time_format_export = "%m_%d_%Y-%H_%M_%S"
-
 
 class UserViewSet(XLSXFileMixin, ReadOnlyModelViewSet):
 	queryset = User.objects.all()
@@ -65,6 +63,7 @@ class UserViewSet(XLSXFileMixin, ReadOnlyModelViewSet):
 		"badge_number": ["exact"],
 		"is_active": ["exact"],
 		"is_staff": ["exact"],
+		"is_facility_manager": ["exact"],
 		"is_superuser": ["exact"],
 		"is_service_personnel": ["exact"],
 		"is_technician": ["exact"],
@@ -74,8 +73,8 @@ class UserViewSet(XLSXFileMixin, ReadOnlyModelViewSet):
 		"access_expiration": ["month", "year", "gte", "gt", "lte", "lt", "isnull"],
 	}
 
-	def get_filename(self):
-		return f"users-{timezone.localtime().strftime(date_time_format_export)}.xlsx"
+	def get_filename(self, *args, **kwargs):
+		return f"users-{export_format_datetime()}.xlsx"
 
 
 class ProjectViewSet(XLSXFileMixin, ReadOnlyModelViewSet):
@@ -89,8 +88,8 @@ class ProjectViewSet(XLSXFileMixin, ReadOnlyModelViewSet):
 		"account_id": ["exact", "in"],
 	}
 
-	def get_filename(self):
-		return f"projects-{timezone.localtime().strftime(date_time_format_export)}.xlsx"
+	def get_filename(self, *args, **kwargs):
+		return f"projects-{export_format_datetime()}.xlsx"
 
 
 class AccountViewSet(XLSXFileMixin, ReadOnlyModelViewSet):
@@ -98,8 +97,8 @@ class AccountViewSet(XLSXFileMixin, ReadOnlyModelViewSet):
 	serializer_class = AccountSerializer
 	filterset_fields = {"id": ["exact", "in"], "name": ["exact"], "active": ["exact"]}
 
-	def get_filename(self):
-		return f"accounts-{timezone.localtime().strftime(date_time_format_export)}.xlsx"
+	def get_filename(self, *args, **kwargs):
+		return f"accounts-{export_format_datetime()}.xlsx"
 
 
 class ToolViewSet(XLSXFileMixin, ReadOnlyModelViewSet):
@@ -116,8 +115,8 @@ class ToolViewSet(XLSXFileMixin, ReadOnlyModelViewSet):
 		"_post_usage_questions": ["isempty"],
 	}
 
-	def get_filename(self):
-		return f"tools-{timezone.localtime().strftime(date_time_format_export)}.xlsx"
+	def get_filename(self, *args, **kwargs):
+		return f"tools-{export_format_datetime()}.xlsx"
 
 
 class AreaViewSet(XLSXFileMixin, ReadOnlyModelViewSet):
@@ -135,8 +134,8 @@ class AreaViewSet(XLSXFileMixin, ReadOnlyModelViewSet):
 		"count_service_personnel_in_occupancy": ["exact"],
 	}
 
-	def get_filename(self):
-		return f"areas-{timezone.localtime().strftime(date_time_format_export)}.xlsx"
+	def get_filename(self, *args, **kwargs):
+		return f"areas-{export_format_datetime()}.xlsx"
 
 
 class ResourceViewSet(XLSXFileMixin, ReadOnlyModelViewSet):
@@ -151,8 +150,8 @@ class ResourceViewSet(XLSXFileMixin, ReadOnlyModelViewSet):
 		"dependent_areas": ["in"],
 	}
 
-	def get_filename(self):
-		return f"resources-{timezone.localtime().strftime(date_time_format_export)}.xlsx"
+	def get_filename(self, *args, **kwargs):
+		return f"resources-{export_format_datetime()}.xlsx"
 
 
 class ReservationViewSet(XLSXFileMixin, ReadOnlyModelViewSet):
@@ -172,8 +171,8 @@ class ReservationViewSet(XLSXFileMixin, ReadOnlyModelViewSet):
 		"question_data": ["isempty"],
 	}
 
-	def get_filename(self):
-		return f"reservations-{timezone.localtime().strftime(date_time_format_export)}.xlsx"
+	def get_filename(self, *args, **kwargs):
+		return f"reservations-{export_format_datetime()}.xlsx"
 
 
 class UsageEventViewSet(XLSXFileMixin, ReadOnlyModelViewSet):
@@ -189,8 +188,8 @@ class UsageEventViewSet(XLSXFileMixin, ReadOnlyModelViewSet):
 		"tool_id": ["exact", "in"],
 	}
 
-	def get_filename(self):
-		return f"usage_events-{timezone.localtime().strftime(date_time_format_export)}.xlsx"
+	def get_filename(self, *args, **kwargs):
+		return f"usage_events-{export_format_datetime()}.xlsx"
 
 
 class AreaAccessRecordViewSet(XLSXFileMixin, ReadOnlyModelViewSet):
@@ -206,8 +205,8 @@ class AreaAccessRecordViewSet(XLSXFileMixin, ReadOnlyModelViewSet):
 		"staff_charge_id": ["exact", "isnull", "in"],
 	}
 
-	def get_filename(self):
-		return f"area_access_records-{timezone.localtime().strftime(date_time_format_export)}.xlsx"
+	def get_filename(self, *args, **kwargs):
+		return f"area_access_records-{export_format_datetime()}.xlsx"
 
 
 class TaskViewSet(XLSXFileMixin, ReadOnlyModelViewSet):
@@ -230,8 +229,8 @@ class TaskViewSet(XLSXFileMixin, ReadOnlyModelViewSet):
 		"resolution_category": ["exact", "in"],
 	}
 
-	def get_filename(self):
-		return f"tasks-{timezone.localtime().strftime(date_time_format_export)}.xlsx"
+	def get_filename(self, *args, **kwargs):
+		return f"tasks-{export_format_datetime()}.xlsx"
 
 
 class ScheduledOutageViewSet(XLSXFileMixin, ReadOnlyModelViewSet):
@@ -248,8 +247,8 @@ class ScheduledOutageViewSet(XLSXFileMixin, ReadOnlyModelViewSet):
 		"resource_id": ["exact", "in", "isnull"],
 	}
 
-	def get_filename(self):
-		return f"outages-{timezone.localtime().strftime(date_time_format_export)}.xlsx"
+	def get_filename(self, *args, **kwargs):
+		return f"outages-{export_format_datetime()}.xlsx"
 
 
 class StaffChargeViewSet(XLSXFileMixin, ReadOnlyModelViewSet):
@@ -263,10 +262,11 @@ class StaffChargeViewSet(XLSXFileMixin, ReadOnlyModelViewSet):
 		"start": ["gte", "gt", "lte", "lt"],
 		"end": ["gte", "gt", "lte", "lt", "isnull"],
 		"validated": ["exact"],
+		"note": ["contains"],
 	}
 
-	def get_filename(self):
-		return f"staff_charges-{timezone.localtime().strftime(date_time_format_export)}.xlsx"
+	def get_filename(self, *args, **kwargs):
+		return f"staff_charges-{export_format_datetime()}.xlsx"
 
 
 class TrainingSessionViewSet(XLSXFileMixin, ReadOnlyModelViewSet):
@@ -284,8 +284,8 @@ class TrainingSessionViewSet(XLSXFileMixin, ReadOnlyModelViewSet):
 		"qualified": ["exact"],
 	}
 
-	def get_filename(self):
-		return f"training_sessions-{timezone.localtime().strftime(date_time_format_export)}.xlsx"
+	def get_filename(self, *args, **kwargs):
+		return f"training_sessions-{export_format_datetime()}.xlsx"
 
 
 class BillingViewSet(XLSXFileMixin, GenericViewSet):
@@ -314,5 +314,5 @@ class BillingViewSet(XLSXFileMixin, GenericViewSet):
 		data.sort(key=lambda x: x.start, reverse=True)
 		return data
 
-	def get_filename(self):
-		return f"billing-{timezone.localtime().strftime(date_time_format_export)}.xlsx"
+	def get_filename(self, *args, **kwargs):
+		return f"billing-{export_format_datetime()}.xlsx"
