@@ -1,6 +1,4 @@
 import io
-from collections import Iterable
-from copy import deepcopy
 from datetime import datetime, timedelta
 from http import HTTPStatus
 from json import dumps, loads
@@ -13,7 +11,7 @@ from dateutil import rrule
 from django.conf import settings
 from django.contrib.auth.decorators import login_required, permission_required
 from django.db.models import Q
-from django.http import HttpResponse, HttpResponseBadRequest, HttpResponseNotFound
+from django.http import HttpResponse, HttpResponseBadRequest, HttpResponseForbidden, HttpResponseNotFound
 from django.shortcuts import get_object_or_404, redirect, render
 from django.template import Context, Template
 from django.utils import timezone
@@ -23,8 +21,11 @@ from django.views.decorators.http import require_GET, require_POST
 from NEMO.decorators import disable_session_expiry_refresh, staff_member_required, synchronized
 from NEMO.exceptions import ProjectChargeException, RequiredUnansweredQuestionsException
 from NEMO.models import (
+	Alert,
 	Area,
 	AreaAccessRecord,
+	Closure,
+	ClosureTime,
 	Configuration,
 	Project,
 	Reservation,
@@ -39,8 +40,10 @@ from NEMO.models import (
 )
 from NEMO.utilities import (
 	EmailCategory,
+	as_timezone,
 	bootstrap_primary_color,
 	create_email_attachment,
+	distinct_qs_value_list,
 	extract_dates,
 	extract_times,
 	format_datetime,
@@ -131,9 +134,9 @@ def calendar(request, item_type=None, item_id=None):
 		'self_login': False,
 		'self_logout': False,
 	}
-	login_logout = get_customization('calendar_login_logout', False)
-	self_login = get_customization('self_log_in', False)
-	self_logout = get_customization('self_log_out', False)
+	login_logout = get_customization('calendar_login_logout', raise_exception=False)
+	self_login = get_customization('self_log_in', raise_exception=False)
+	self_logout = get_customization('self_log_out', raise_exception=False)
 	if login_logout == 'enabled':
 		dictionary['self_login'] = self_login == 'enabled'
 		dictionary['self_logout'] = self_logout == 'enabled'
@@ -352,12 +355,16 @@ def create_item_reservation(request, current_user, start, end, item_type: Reserv
 	setattr(new_reservation, item_type.value, item)
 	new_reservation.start = start
 	new_reservation.end = end
-	new_reservation.short_notice = determine_insufficient_notice(item, start) if item_type == ReservationItemType.TOOL else False
+	new_reservation.short_notice = item.determine_insufficient_notice(start) if item_type == ReservationItemType.TOOL else False
 	policy_problems, overridable = check_policy_to_save_reservation(cancelled_reservation=None, new_reservation=new_reservation, user_creating_reservation=request.user, explicit_policy_override=explicit_policy_override)
 
 	# If there was a policy problem with the reservation then return the error...
 	if policy_problems:
-		return render(request, 'calendar/policy_dialog.html', {'policy_problems': policy_problems, 'overridable': overridable and request.user.is_staff, 'reservation_action': 'create'})
+		return render(request, 'calendar/policy_dialog.html', {
+			'policy_problems': policy_problems,
+			'overridable': overridable and request.user.is_staff,
+			'reservation_action': 'create'
+		})
 
 	# All policy checks have passed.
 
@@ -371,7 +378,10 @@ def create_item_reservation(request, current_user, start, end, item_type: Reserv
 			try:
 				new_reservation.project = Project.objects.get(id=request.POST['project_id'])
 			except:
-				return render(request, 'calendar/project_choice.html', {'active_projects': active_projects})
+				return render(request, 'calendar/project_choice.html', {
+					'active_projects': active_projects,
+					'missed_reservation_threshold': new_reservation.reservation_item.missed_reservation_threshold
+				})
 
 		# Check if we are allowed to bill to project
 		try:
@@ -536,7 +546,7 @@ def create_outage(request):
 		elif submitted_frequency == 'DAILY_WEEKENDS':
 			by_week_day = (rrule.SA, rrule.SU)
 		frequency = recurrence_frequencies.get(submitted_frequency, rrule.DAILY)
-		rules: Iterable[datetime] = rrule.rrule(dtstart=start, freq=frequency, interval=int(request.POST.get('recurrence_interval',1)), until=date_until, byweekday=by_week_day)
+		rules = rrule.rrule(dtstart=start, freq=frequency, interval=int(request.POST.get('recurrence_interval',1)), until=date_until, byweekday=by_week_day)
 		for rule in list(rules):
 			recurring_outage = ScheduledOutage()
 			recurring_outage.creator = outage.creator
@@ -614,28 +624,13 @@ def modify_reservation(request, start_delta, end_delta):
 		pass
 	# Record the current time so that the timestamp of the cancelled reservation and the new reservation match exactly.
 	now = timezone.now()
-	# Create a new reservation for the user.
-	new_reservation = Reservation()
-	new_reservation.title = reservation_to_cancel.title
-	new_reservation.creator = request.user
-	new_reservation.additional_information = reservation_to_cancel.additional_information
-	# A change in start time will only be provided if the reservation is being moved.
-	new_reservation.start = reservation_to_cancel.start
-	new_reservation.self_configuration = reservation_to_cancel.self_configuration
-	new_reservation.short_notice = False
-	if start_delta:
-		new_reservation.start += start_delta
-	if new_reservation.self_configuration:
-		# Reservation can't be short notice since the user is configuring the tool themselves.
-		new_reservation.short_notice = False
-	elif new_reservation.tool:
-		new_reservation.short_notice = determine_insufficient_notice(reservation_to_cancel.tool, new_reservation.start)
-	# A change in end time will always be provided for reservation move and resize operations.
-	new_reservation.end = reservation_to_cancel.end + end_delta
-	new_reservation.reservation_item = reservation_to_cancel.reservation_item
-	new_reservation.project = reservation_to_cancel.project
-	new_reservation.user = reservation_to_cancel.user
+	# Create a new reservation for the user by copying the original one.
+	new_start = reservation_to_cancel.start + start_delta if start_delta else None
+	new_end = reservation_to_cancel.end + end_delta if end_delta else None
+	new_reservation = reservation_to_cancel.copy(new_start, new_end)
+	# Set new creator/time
 	new_reservation.creation_time = now
+	new_reservation.creator = request.user
 
 	response = check_policy_to_cancel_reservation(request.user, reservation_to_cancel, new_reservation)
 	# Do not move the reservation if the user was not authorized to cancel it.
@@ -676,16 +671,6 @@ def modify_outage(request, start_delta, end_delta):
 	return HttpResponse()
 
 
-def determine_insufficient_notice(tool, start):
-	""" Determines if a reservation is created that does not give
-	the staff sufficient advance notice to configure a tool. """
-	for config in tool.configuration_set.all():
-		advance_notice = start - timezone.now()
-		if advance_notice < timedelta(hours=config.advance_notice_limit):
-			return True
-	return False
-
-
 @login_required
 @require_POST
 def cancel_reservation(request, reservation_id):
@@ -732,9 +717,26 @@ def change_reservation_project(request, reservation_id):
 	""" Change reservation project for a user. """
 	reservation = get_object_or_404(Reservation, id=reservation_id)
 	project = get_object_or_404(Project, id=request.POST['project_id'])
-	if (request.user.is_staff or request.user == reservation.user) and reservation.has_not_ended() and reservation.has_not_started() and  project in reservation.user.active_projects():
+	try:
+		check_billing_to_project(project, reservation.user, reservation.reservation_item)
+	except ProjectChargeException as e:
+		return HttpResponseBadRequest(e.msg)
+
+	if (request.user.is_staff or request.user == reservation.user) and \
+		reservation.has_not_ended() and reservation.has_not_started() and \
+		project in reservation.user.active_projects():
 		reservation.project = project
 		reservation.save()
+	else:
+		# project for reservation was not eligible to be changed
+		if not (request.user.is_staff or request.user == reservation.user):
+			return HttpResponseForbidden(f"{request.user} is not authorized to change the project for this reservation")
+		if not reservation.has_not_ended():
+			return HttpResponseBadRequest("Project cannot be changed; reservation has already ended")
+		if not reservation.has_not_started():
+			return HttpResponseBadRequest("Project cannot be changed; reservation has already started")
+		if project not in reservation.user.active_projects():
+			return HttpResponseForbidden(f"{project} is not one of {reservation.user}'s active projects")
 	return HttpResponse()
 
 
@@ -873,38 +875,6 @@ def send_email_usage_reminders(projects_to_exclude=None):
 	return HttpResponse()
 
 
-@login_required
-@require_GET
-def reservation_details(request, reservation_id):
-	reservation = get_object_or_404(Reservation, id=reservation_id)
-	if reservation.cancelled:
-		error_message = 'This reservation was cancelled by {0} at {1}.'.format(reservation.cancelled_by, format_datetime(reservation.cancellation_time))
-		return HttpResponseNotFound(error_message)
-	reservation_project_can_be_changed = (request.user.is_staff or request.user == reservation.user) and reservation.has_not_ended and reservation.has_not_started and reservation.user.active_project_count() > 1
-	return render(request, 'calendar/reservation_details.html', {'reservation': reservation, 'reservation_project_can_be_changed': reservation_project_can_be_changed})
-
-
-@login_required
-@require_GET
-def outage_details(request, outage_id):
-	outage = get_object_or_404(ScheduledOutage, id=outage_id)
-	return render(request, 'calendar/outage_details.html', {'outage': outage})
-
-
-@login_required
-@require_GET
-def usage_details(request, event_id):
-	event = get_object_or_404(UsageEvent, id=event_id)
-	return render(request, 'calendar/usage_details.html', {'event': event})
-
-
-@login_required
-@require_GET
-def area_access_details(request, event_id):
-	event = get_object_or_404(AreaAccessRecord, id=event_id)
-	return render(request, 'calendar/area_access_details.html', {'event': event})
-
-
 @login_required
 @require_GET
 @permission_required('NEMO.trigger_timed_services', raise_exception=True)
@@ -959,8 +929,8 @@ def do_cancel_unused_reservations():
 			# Staff may abandon reservations.
 			if r.user.is_staff:
 				continue
-			# if there was no area access starting or ending since the threshold timestamp then we assume the reservation was missed
-			if not (AreaAccessRecord.objects.filter(area__id=area.id, customer=r.user, start__gte=threshold).exists() or AreaAccessRecord.objects.filter(area__id=area.id, customer=r.user, end__gte=threshold).exists()):
+			# if the user is not already logged in or if there was no area access starting or ending since the threshold timestamp then we assume the reservation was missed
+			if not (AreaAccessRecord.objects.filter(area__id=area.id, customer=r.user, staff_charge=None, end=None).exists() or AreaAccessRecord.objects.filter(area__id=area.id, customer=r.user, start__gte=threshold).exists() or AreaAccessRecord.objects.filter(area__id=area.id, customer=r.user, end__gte=threshold).exists()):
 				# Mark the reservation as missed and notify the user & staff.
 				r.missed = True
 				r.save()
@@ -1063,15 +1033,12 @@ def shorten_reservation(user: User, item: Union[Area, Tool], new_end: datetime =
 	try:
 		if new_end is None:
 			new_end = timezone.now()
-		current_reservation = Reservation.objects.filter(start__lt=timezone.now(), end__gt=timezone.now(),
-														 cancelled=False, missed=False, shortened=False, user=user)
-		current_reservation = current_reservation.get(**{ReservationItemType.from_item(item).value: item})
+		current_reservation_qs = Reservation.objects.filter(start__lt=timezone.now(), end__gt=timezone.now(),
+															cancelled=False, missed=False, shortened=False, user=user)
+		current_reservation = current_reservation_qs.get(**{ReservationItemType.from_item(item).value: item})
 		# Staff are exempt from mandatory reservation shortening.
 		if user.is_staff is False:
-			new_reservation = deepcopy(current_reservation)
-			new_reservation.id = None
-			new_reservation.pk = None
-			new_reservation.end = new_end
+			new_reservation = current_reservation.copy(new_end=new_end)
 			new_reservation.save()
 			current_reservation.shortened = True
 			current_reservation.descendant = new_reservation
@@ -1210,3 +1177,69 @@ def create_ics_for_reservation(reservation: Reservation, cancelled=False):
 
 	attachment = create_email_attachment(ics, maintype='text', subtype='calendar', method=method_name)
 	return attachment
+
+
+@login_required
+@require_GET
+@permission_required("NEMO.trigger_timed_services", raise_exception=True)
+def create_closure_alerts(request):
+	return do_create_closure_alerts()
+
+
+def do_create_closure_alerts():
+	future_times = ClosureTime.objects.filter(closure__alert_days_before__isnull=False, end_time__gt=timezone.now())
+	for closure_time in future_times:
+		create_alert_for_closure_time(closure_time)
+	for closure in Closure.objects.filter(notify_managers_last_occurrence=True):
+		closure_time_ending = ClosureTime.objects.filter(closure=closure).latest("end_time")
+		if as_timezone(closure_time_ending.end_time).date() == timezone.now().date():
+			email_last_closure_occurrence(closure_time_ending)
+	return HttpResponse()
+
+
+def create_alert_for_closure_time(closure_time: ClosureTime):
+	# Create alerts a week before their debut time (closure start - days before) at the latest
+	now = timezone.now()
+	next_week = now + timedelta(weeks=1)
+	closure = closure_time.closure
+	alert_start = closure_time.start_time - timedelta(days=closure.alert_days_before)
+	time_ready = alert_start <= next_week
+	if time_ready:
+		# Check if there is already an alert with the same title ending at the closure end time
+		alert_already_exist = Alert.objects.filter(title=closure.name, expiration_time=closure_time.end_time).exists()
+		if not alert_already_exist:
+			areas = Area.objects.filter(id__in=distinct_qs_value_list(closure.physical_access_levels.all(), "area"))
+			dictionary = {
+				"closure_time": closure_time,
+				"name": closure.name,
+				"staff_absent": closure.staff_absent,
+				"start_time": closure_time.start_time,
+				"end_time": closure_time.end_time,
+				"areas": areas,
+			}
+			contents = Template(closure.alert_template).render(Context(dictionary)) if closure.alert_template else None
+			Alert.objects.create(
+				title=closure.name,
+				contents=contents,
+				category="Closure",
+				debut_time=alert_start,
+				expiration_time=closure_time.end_time,
+			)
+
+
+def email_last_closure_occurrence(closure_time):
+	facility_manager_emails = User.objects.filter(is_active=True, is_facility_manager=True).values_list(
+		"email", flat=True
+	)
+	message = f"""
+Dear facility manager,<br>
+This email is to inform you that today was the last occurrence for the {closure_time.closure.name} facility closure.
+<br><br>Go to NEMO -> Detailed administration -> Closures to add more times if needed.
+"""
+	send_mail(
+		subject=f"Last {closure_time.closure.name} occurrence",
+		content=message,
+		from_email=settings.SERVER_EMAIL,
+		to=facility_manager_emails,
+		email_category=EmailCategory.SYSTEM,
+	)
