@@ -3,7 +3,7 @@ from django.shortcuts import render
 from django.template import Template, Context
 from django.views.decorators.http import require_http_methods
 
-from NEMO.utilities import parse_parameter_string, send_mail
+from NEMO.utilities import parse_parameter_string, send_mail, EmailCategory
 from NEMO.views.constants import FEEDBACK_MAXIMUM_LENGTH
 from NEMO.views.customization import get_customization, get_media_file_contents
 
@@ -27,7 +27,7 @@ def feedback(request):
 	}
 
 	email = Template(email_contents).render(Context(dictionary))
-	send_mail('Feedback from ' + str(request.user), email, request.user.email, [recipient])
+	send_mail(subject='Feedback from ' + str(request.user), content=email, from_email=request.user.email, to=[recipient], email_category=EmailCategory.FEEDBACK)
 	dictionary = {
 		'title': 'Feedback',
 		'heading': 'Thanks for your feedback!',
