@@ -1,20 +1,41 @@
 from collections import defaultdict
-from datetime import timedelta, date, datetime
-from typing import Optional, List, Union
+from datetime import date, datetime, timedelta
+from typing import List, Optional, Union
 
 from django.db.models import Q
 from django.http import HttpResponse, HttpResponseBadRequest
-from django.template import Template, Context
+from django.template import Context, Template
 from django.utils import timezone
-from django.utils.formats import localize
 
-from NEMO.exceptions import InactiveUserError, NoActiveProjectsForUserError, PhysicalAccessExpiredUserError, \
-	NoPhysicalAccessUserError, NoAccessiblePhysicalAccessUserError, UnavailableResourcesUserError, \
-	MaximumCapacityReachedError, ReservationRequiredUserError, ScheduledOutageInProgressError, \
-	NotAllowedToChargeProjectException, ItemNotAllowedForProjectException, ProjectChargeException
-from NEMO.models import Reservation, AreaAccessRecord, ScheduledOutage, User, Area, PhysicalAccessLevel, \
-	ReservationItemType, Tool, Project, PhysicalAccessException, Consumable, StaffCharge
-from NEMO.utilities import format_datetime, send_mail, EmailCategory, distinct_qs_value_list
+from NEMO.exceptions import (
+	InactiveUserError,
+	ItemNotAllowedForProjectException,
+	MaximumCapacityReachedError,
+	NoAccessiblePhysicalAccessUserError,
+	NoActiveProjectsForUserError,
+	NoPhysicalAccessUserError,
+	NotAllowedToChargeProjectException,
+	PhysicalAccessExpiredUserError,
+	ProjectChargeException,
+	ReservationRequiredUserError,
+	ScheduledOutageInProgressError,
+	UnavailableResourcesUserError,
+)
+from NEMO.models import (
+	Area,
+	AreaAccessRecord,
+	ClosureTime,
+	Consumable,
+	PhysicalAccessLevel,
+	Project,
+	Reservation,
+	ReservationItemType,
+	ScheduledOutage,
+	StaffCharge,
+	Tool,
+	User,
+)
+from NEMO.utilities import EmailCategory, distinct_qs_value_list, format_daterange, format_datetime, send_mail
 from NEMO.views.customization import get_customization, get_media_file_contents
 
 
@@ -210,13 +231,13 @@ def check_policy_to_save_reservation(cancelled_reservation: Optional[Reservation
 	# An explicit policy override allows this rule to be broken.
 	extension_of_area_reservation = new_reservation.area and cancelled_reservation and cancelled_reservation.start == new_reservation.start
 	if not extension_of_area_reservation and new_reservation.start < timezone.now():
-		policy_problems.append("Reservation start time (" + format_datetime(new_reservation.start) + ") is earlier than the current time (" + format_datetime(timezone.now()) + ").")
+		policy_problems.append("Reservation start time (" + format_datetime(new_reservation.start) + ") is earlier than the current time (" + format_datetime() + ").")
 
 	# The user may not move or resize a reservation to have an end time that is earlier than the current time.
 	# Staff may break this rule.
 	# An explicit policy override allows this rule to be broken.
 	if new_reservation.end < timezone.now():
-		policy_problems.append("Reservation end time (" + format_datetime(new_reservation.end) + ") is earlier than the current time (" + format_datetime(timezone.now()) + ").")
+		policy_problems.append("Reservation end time (" + format_datetime(new_reservation.end) + ") is earlier than the current time (" + format_datetime() + ").")
 
 	# The user must be qualified on the tool in question in order to create, move, or resize a reservation.
 	# Staff may break this rule.
@@ -233,12 +254,13 @@ def check_policy_to_save_reservation(cancelled_reservation: Optional[Reservation
 	if item_type == ReservationItemType.AREA:
 		user_access_levels = user.accessible_access_levels_for_area(new_reservation.area)
 		if not any([access_level.accessible_at(new_reservation.start) for access_level in user_access_levels]) or not any([access_level.accessible_at(new_reservation.end) for access_level in user_access_levels]):
-			# it could be inaccessible because of an ongoing exception at the start or end time
-			first_access_exception: PhysicalAccessException = next(iter([access_level.ongoing_exception(new_reservation.start) for access_level in user_access_levels]), None)
-			if not first_access_exception:
-				first_access_exception = next(iter([access_level.ongoing_exception(new_reservation.end) for access_level in user_access_levels]), None)
-			if first_access_exception:
-				details = f" due to the following exception: {first_access_exception.name} (from {localize(first_access_exception.start_time.astimezone(timezone.get_current_timezone()))} to {localize(first_access_exception.end_time.astimezone(timezone.get_current_timezone()))}"
+			# it could be inaccessible because of an ongoing closure at the start or end time
+			first_closure_time: ClosureTime = next(iter([access_level.ongoing_closure_time(new_reservation.start) for access_level in user_access_levels]), None)
+			if not first_closure_time:
+				# if there is no closure at start, check at end time
+				first_closure_time = next(iter([access_level.ongoing_closure_time(new_reservation.end) for access_level in user_access_levels]), None)
+			if first_closure_time:
+				details = f" due to the following closure: {first_closure_time.closure.name} ({format_daterange(first_closure_time.start_time, first_closure_time.end_time)})"
 			# or simply due to scheduling
 			else:
 				details = f" (times allowed in this area are: {', '.join([access.get_schedule_display_with_times() for access in user_access_levels])})" if user_access_levels else ''
@@ -326,7 +348,7 @@ def check_coincident_item_reservation_policy(cancelled_reservation: Optional[Res
 				# Check only distinct users since the same user could make reservations in different rooms
 				maximum_users, time = maximum_users_in_overlapping_reservations(reservations)
 				if maximum_users > area.maximum_capacity:
-					time_display = "at this time" if time is None else "at "+timezone.localtime(time).strftime('%I:%M %p')
+					time_display = "at this time" if time is None else "at "+ format_datetime(time, "TIME_FORMAT")
 					policy_problems.append(f"The {area} would be over its maximum capacity {time_display}. Please choose a different time.")
 
 	# The user may not create, move, or resize a reservation to coincide with a scheduled outage.
@@ -538,8 +560,8 @@ def check_policy_to_enter_this_area(area:Area, user:User):
 	else:
 		# Check if the user normally has access to this area door at the current time (or access to any parent)
 		if not any([access_level.accessible() for access_level in user.accessible_access_levels_for_area(area)]):
-			first_access_exception = next(iter([access_level.ongoing_exception() for access_level in user.accessible_access_levels_for_area(area)]), None)
-			raise NoAccessiblePhysicalAccessUserError(user=user, area=area, access_exception=first_access_exception)
+			first_closure_time = next(iter([access_level.ongoing_closure_time() for access_level in user.accessible_access_levels_for_area(area)]), None)
+			raise NoAccessiblePhysicalAccessUserError(user=user, area=area, closure_time=first_closure_time)
 
 	if not user.is_staff and not user.is_service_personnel:
 		for a in area.get_ancestors(ascending=True, include_self=True):
